import java . math . BigInteger;
import java . io . FileReader;
import java . util . Hashtable;



class Program implements BigIntegerConstants
{



	private Instruction [] instructions;
	private int length;



	static Program create (BigInteger number)
	{
		return new Program (number);
	}



	private Program (BigInteger pcode)
	{
		pcode = pcode . add (ONE);

		length = pcode . bitCount ();
		instructions = new Instruction [length + 1];

		instructions [0] = null;

		int instruction_count = 1;
		while (true)
		{
			int icode = pcode . getLowestSetBit ();
			if (icode < 0)
				break;
			instructions [instruction_count ++] = Instruction . create (BigInteger . valueOf (icode));
			pcode = pcode . shiftRight (icode + 1);
		}
	}



	static Program create (java . io . File file)
		throws
			java . io . FileNotFoundException,
			java . io . IOException,
			CompileError
	{
		ProgramReader reader = new ProgramReader (file);
		return reader . read_program (new Program ());
	}



	private Program ()
	{
		instructions = new Instruction [1];
		length = 0;
	}



	void append_instruction (Instruction instruction)
	{
		if (length == instructions . length - 1)
		{
			Instruction [] new_array = new Instruction [2 * instructions . length];
			System . arraycopy (instructions, 0, new_array, 0, instructions . length);
			instructions = new_array;
		}
		instructions [++ length] = instruction;
	}



	Instruction fetch_instruction (int pc)
	{
		if (pc > length)
			pc = 0;
		return instructions [pc];
	}



	void print (java . io . PrintStream out)
	{
		for (int i = 1; i <= length; ++ i)
			out . println (i + ": " + instructions [i] + "   [#" + instructions [i] . get_code_number () + "]");
	}



	BigInteger get_code_number ()
	{
		BigInteger pcode = BigInteger . valueOf (0);
		for (int i = length; i > 0; -- i)
		{
			BigInteger icode = instructions [i] . get_code_number ();
			int temp = NaturalMath . intValue (icode);
			pcode = pcode . shiftLeft (temp + 1) . add (TWO . pow (temp));
		}
		return pcode . subtract (ONE);
	}



}



