import java . io . IOException;
import java . math . BigInteger;



class TokenReader
{



	private final CharacterReader reader;



	TokenReader (java . io . File file)
		throws java . io . FileNotFoundException
	{
		reader = new CharacterReader (file);
		peeked_token = null;
	}



	void reset () throws IOException
	{
		reader . reset ();
		peeked_token = null;
	}



	private boolean is_label_begin_char (char c)
	{
		return Character . isLetter (c) || c == '_';
	}



	private boolean is_label_middle_char (char c)
	{
		return is_label_begin_char (c) || Character . isDigit (c);
	}



	private String get_alnum_word () throws IOException
	{
		StringBuffer buf = new StringBuffer ();
		while (true)
		{
			if (! is_label_middle_char (reader . peek_char ()))
				break;
			buf . append (reader . read_char ());
		}
		return buf . toString ();
	}



	private BigInteger get_integer () throws IOException
	{
		StringBuffer buf = new StringBuffer ();
		while (true)
		{
			if (! Character . isDigit (reader . peek_char ()))
				break;
			buf . append (reader . read_char ());
		}
		return new BigInteger (buf . toString ());
	}



	private Token get_token () throws IOException
	{
		char c;
		while (true)
		{
			c = reader . peek_char ();
			if (! Character . isWhitespace (c))
				break;
			reader . read_char ();
		}

		int line_number = reader . get_line_number ();
		if (is_label_begin_char (c))
			return Token . create (get_alnum_word (), line_number);
		else if (Character . isDigit (c))
			return Token . create (get_integer (), line_number);
		else
			return Token . create (reader . read_char (), line_number);
	}



	private Token peeked_token;



	Token peek_token () throws IOException
	{
		if (peeked_token == null)
			peeked_token = get_token ();
		return peeked_token;
	}



	Token read_token () throws IOException
	{
		if (peeked_token != null)
		{
			Token t = peeked_token;
			peeked_token = null;
			return t;
		}
		return get_token ();
	}



}



