import java . math . BigInteger;



class urm implements BigIntegerConstants
{



	private static final BigInteger UNKNOWN = BigInteger . valueOf (-1);



	private static void print_usage_info ()
	{
		System . out . println ("usage: java urm (<program number> | <program filename>) <R1 input> <R2 input> ... [time <steps>]");
	}



	private static boolean is_all_digits (String s)
	{
		for (int i = 0; i < s . length (); ++ i)
			if (! Character . isDigit (s . charAt (i)))
				return false;
		return true;
	}



	public static void main (String [] args) throws Exception
	{
		// Output command-line help if no parameters were supplied:
		if (args . length < 1)
		{
			print_usage_info ();
			return;
		}

		// Load URM program by program number or program filename:
		BigInteger program_number;
		Program program;
		if (is_all_digits (args [0]))
		{
			System . out . print ("Decoding program...");
			program_number = new BigInteger (args [0]);
			program = Program . create (program_number);
			System . out . println ("done.");
		}
		else
		{
			program = Program . create (new java . io . File (args [0]));
			program_number = UNKNOWN;
		}

		// Initialize register bank with input values from the command line:
		RegisterBank registers = new RegisterBank ();
		int inputs = 0;
		int param = 1;
		while (param < args . length && is_all_digits (args [param]))
		{
			registers . store (BigInteger . valueOf (param), new BigInteger (args [param]));
			++ inputs;
			++ param;
		}

		int time = -1;
		while (param < args . length)
		{
			// Set a limit on the number of program steps if a limit is specified:
			if (args [param] . equals ("time"))
			{
				++ param;
				if (param == args . length || ! is_all_digits (args [param]))
					exit_with_error ("Steps option must be followed by an integer.");
				time = Integer . parseInt (args [param]);
				++ param;
			}
			else
				exit_with_error ("Unrecognized option \"" + args [param] + "\".");
		}

		// Configure URM processor:
		Processor processor = new Processor ();
		processor . program = program;
		processor . registers = registers;
		processor . pc = 1;

		// Run program:
		System . out . print ("Executing program...");
		int result = processor . run (time);
		if (result >= 0)
			System . out . println ("done.");
		else
			System . out . println ("aborted.");

		// Prepare input_string for report:
		StringBuffer input_string = new StringBuffer ();
		if (inputs == 0)
			input_string . append ("none");
		else
		{
			input_string . append ("(");
			param = 1;
			while (true)
			{
				input_string . append (args [param]);
				++ param;
				if (param > inputs)
					break;
				input_string . append (", ");
			}
			input_string . append (")");
		}

		// Prepare output string for report:
		String output_string;
		if (result >= 0)
			output_string = registers . load (ONE) . toString ();
		else
			output_string = "unknown";

		// Prepare time_string for report:
		String time_string;
		if (result >= 0)
			time_string = result + " steps";
		else
			time_string = "not less than " + time + " steps";

		// Output report:
		System . out . print
		(
			"\n" +
			"Execution summary:\n" +
			"------------------\n" +
			"program: " + args [0] + "\n" +
			"  input: " + input_string + "\n" +
			" output: " + output_string + "\n" +
			"   time: " + time_string + "\n" +
			"\n" +
			"Program listing:\n" +
			"----------------\n"
		);
		program . print (System . out);
		System . out . print
		(
			"\n" +
			"Program code:\n" +
			"-------------\n"
		);
		if (program_number == UNKNOWN)
			program_number = program . get_code_number ();
		System . out . println (program_number . toString ());
	}



	private static void exit_with_error (String msg)
	{
		System . out . println ("java urm: Error: " + msg);
		System . exit (-1);
	}



}



