﻿using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace com.scrigner.RAMModel.Test
{
	// The following are NOT strict covering unit tests. They are just meant to test a few features and to show some code examples
	[TestClass]
	public class SomeTests
	{
		// ==========================================================================================
		// LoopDetection
		// ==========================================================================================
		[TestMethod]
		[ExpectedException(typeof(CLoopDetectedException))]
		public void LoopDetection()
		{
			IProgram program = new CProgram(
									new IStatement[]
									{
										/* 01 */ new CStatementCondition(1)	// == C(1,1,1)
									});
			long lResult = new CMachine().Execute(program);
		}


		// ==========================================================================================
		// XMinusOne Tests
		// ==========================================================================================
		[TestMethod]
		public void XMinus1_WhenXIsGreaterThanOne()
		{
			IProgram program = new CProgramXMinus1();

			long x = 5;
			long lResult = new CMachine(new long[]{x}).Execute(program);

			Assert.AreEqual(4, lResult);
		}
		[TestMethod]
		public void XMinus1_WhenXIsEqualToOne()
		{
			try
			{
				IProgram program = new CProgramXMinus1();

				long x = 1;
				long lResult = new CMachine(new long[]{x}).Execute(program);

				Assert.AreEqual(0, lResult);
			}
			catch (CLoopDetectedException ex)
			{}
		}
		[TestMethod]
		public void XMinus1_WhenXIsLessThanOne()
		{
			IProgram program = new CProgramXMinus1();

			long x = 0;
			long lResult = new CMachine(new long[]{x}).Execute(program);

			Assert.AreEqual(0, lResult);
		}
		// ==========================================================================================
		// XMinusY Version A Tests
		// ==========================================================================================
		[TestMethod]
		public void XMinusY_VersionA_WhenXIsGreaterThanY()
		{
			IProgram program = new CProgramXMinusY_VersionA();

			long x = 5;
			long y = 3;
			long lResult = new CMachine(new long[]{x, y}).Execute(program);

			Assert.AreEqual(2, lResult);
		}
		[TestMethod]
		public void XMinusY_VersionA_WhenXIsEqualToY()
		{
			IProgram program = new CProgramXMinusY_VersionA();

			long x = 5;
			long y = 5;
			long lResult = new CMachine(new long[]{x, y}).Execute(program);

			Assert.AreEqual(0, lResult);
		}
		[TestMethod]
		[ExpectedException(typeof(CLoopDetectedException))]
		public void XMinusY_VersionA_WhenXIsLessThanY()
		{
			IProgram program = new CProgramXMinusY_VersionA();

			long x = 5;
			long y = 7;
			long lResult = new CMachine(new long[]{x, y}).Execute(program);
		}



	}
}
