﻿using System.Collections.Generic;

namespace com.scrigner.RAMModel
{
	// ================================================================================
	// A program is an ordered list of instructions.
	// Instructions are easily extendable (strategy pattern).
	// We might need some pseudo instructions in order to load values on the tape to reach the machine initial condition.
	// ================================================================================
	public abstract class CAbstractProgram : IProgram
	{
		private readonly List<IStatement> zStatements;
		// ================================================================================
		// CTORs
		// ================================================================================
		public CAbstractProgram(IEnumerable<IStatement> statements)	// Zero statements allowed
		{
			CContract.AssertIsNotNull("statements", statements);

			zStatements = new List<IStatement>(statements);
		}
		// ================================================================================
		// Properties
		// ================================================================================
		public IEnumerable<IStatement> Instructions	{get{foreach (IStatement x in zStatements){yield return x;}}}
		//public int LastPosition	{get{return zStatements.Count;}}

		// ================================================================================
		// The program needs to export statements by position to the machine and to allow clients to verify if we reached the end of the program
		// ================================================================================
		public bool IsStatementDefinedAt(int iPosition)
		{
			CContract.Assert(iPosition >= 1, "iPosition >= 1");

			return iPosition <= zStatements.Count;
		}
		public IStatement GetStatement(int iPosition)		// Here we should use and IMaybe<IStatement> as return value, which we don't have in this project
		{
			CContract.Assert(iPosition >= 1, "iPosition >= 1");
			CContract.Assert(this.IsStatementDefinedAt(iPosition), "this.IsStatementAt(iPosition)");
			
			return zStatements[iPosition-1];
		}
	}
}
