﻿
using System.Text;
namespace com.scrigner.RAMModel
{
	// ================================================================================
	// Since the Machine is immutable, the current state is hold in a stack variable during the Execute method processing.
	// In order to notify the machine state (which is the machine immutable state united to the current value of the program counter), 
	// we must create a new MachineState
	// ================================================================================
	public class CMachineState : IMachineState
	{
		private readonly int ziStepIndex;				// 1,2,3.... 
		private readonly int ziNextStatementIndex;
		private readonly IMachine zPMachine;			// Here the ITape might have been enough ...
		private readonly IProgram zPProgram;

		public CMachineState(int iStepIndex, int iNextStatementIndex, IMachine machine, IProgram program) 
		{
			CContract.Assert(iStepIndex >= 0, "iStepIndex >= 0");
			CContract.Assert(iNextStatementIndex >= 1, "iNextStatementIndex >= 1");
			CContract.AssertIsNotNull("machine", machine);
			CContract.AssertIsNotNull("program", program);

			ziStepIndex = iStepIndex;
			ziNextStatementIndex = iNextStatementIndex;
			zPMachine = machine;
			zPProgram = program;
		}
		public int StepIndex			{get{return ziStepIndex;}}
		public int NextStatementIndex	{get{return ziNextStatementIndex;}}
		public IMachine Machine			{get{return zPMachine;}}
		public IProgram Program			{get{return zPProgram;}}

		public string GetAsText()
		{
			StringBuilder sb = new StringBuilder();

			sb.Append(ziStepIndex.ToString("00000"));
			sb.Append(" ");
			sb.Append(ziNextStatementIndex.ToString("00000"));
			sb.Append(" ");
			if (zPProgram.IsStatementDefinedAt(ziNextStatementIndex))
			{
				sb.Append(zPProgram.GetStatement(ziNextStatementIndex).GetAsText().PadRight(12, ' '));
			}
			else
			{
				sb.Append("".PadRight(12, ' '));
			}

			foreach (long value in zPMachine.Tape.Values)
			{
				sb.Append(value.ToString("000"));
				sb.Append(" ");
			}
			return sb.ToString();

		}
	}
}
