﻿using System.Collections.Generic;

namespace com.scrigner.RAMModel
{
	// ================================================================================
	// The Tape class represent the URM Tape, which is semi infinite on the right and starts at position 1.
	// We internally represent the tape as a list. The list-index <-> square-position conversion (+- 1) il a tape responsability
	// ================================================================================
	// We decided to use a long to represent values and an int for positions of tape squares
	// instead of the unsigned ulong and uint, to remain cross language compliant.
	// We also call 
	//	- "Position" the Tape index + 1
	//	- "Value" the content of a Square
	// ================================================================================
	public class CTape : ITape 
	{
		private readonly List<long> zSquares;
		// ================================================================================
		// CTORs
		// ================================================================================
		public CTape() : this (new long[]{}) {}
		public CTape(IEnumerable<long> initialValues)
		{
			zSquares = new List<long>(initialValues);
		}
		// ================================================================================
		// Values - for debugging only : touched squares only
		// ================================================================================
		public IEnumerable<long> Values	{get{foreach (long x in zSquares){yield return x;}}}	
		// ================================================================================
		// SetValue, GetValue
		// ================================================================================
		public void SetValue(long value, int iPosition)
		{
			CContract.Assert(iPosition >= 1, "iPosition >= 1");		// Position must be GE 1
			CContract.Assert(value >= 0, "value >= 0");				// Value must be Natural
			
			// ... we extend the tape to iPosition, in needed
			this.ExtendTo(iPosition);
			// ... and we set the value on the tape
			zSquares[iPosition-1] = value;
		}
		public long GetValue(int iPosition)
		{	
			CContract.Assert(iPosition >= 1, "iPosition >= 1");
			
			// ... we extend the tape to iPosition, in needed
			this.ExtendTo(iPosition);
			// ... and we get and return the value from the tape
			return zSquares[iPosition-1];
		}
		// ================================================================================
		// Privates
		// ================================================================================
		// Since the tape is infinite, we lazily extend the underlying list to the just touched iPosition
		// The "extension" is executed both on GetValue and on SetValue
		// ================================================================================
		private void ExtendTo(int iPosition)
		{
			CContract.Assert(iPosition >= 1, "iPosition >= 1");

			while (zSquares.Count < iPosition)
			{
				zSquares.Add(0);
			}
		}

	}
}
