﻿using com.scrigner.RAMModel.Parsing;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;

namespace com.scrigner.RAMModel
{
	class Program
	{
		static void Main(string[] args)
		{
			if (args.Length >= 1)
			{
				try
				{	// Program
					string source = args[0];
					if (File.Exists(args[0]))
					{
						source = File.ReadAllText(args[0]);
					}
					// Initial values
					List<long> initialValues = new List<long>();
					for (int i = 1; i<args.Length; i++)
					{
						initialValues.Add((int)uint.Parse(args[i]));
					}

					try
					{	// We parse the text into a program ...
						IProgram program = new CProgramParser().Parse(source);
						// ... we create a machine with the initial values retieved from args ...
						IMachine machine = new CMachine(initialValues);
						machine.NotifyProcessingState += machine_NotifyProcessingState;
						long result = machine.Execute(program);

						Console.WriteLine("Execution Completed. Result=[" + result + "]");
					}
					catch (CLoopDetectedException ex)
					{
						Console.WriteLine("Execution halted in a loop.");
					}

				}
				catch (Exception ex)
				{
					Console.WriteLine(ex.ToString());
				}
			}
			else
			{
				Console.WriteLine("USAGE > com.scrigner.URM Program v1 v2 v3 .... vi");
				Console.WriteLine("Program can be inline or a file path. Vi are the initial arguments.");
			}
		}

		static void machine_NotifyProcessingState(object sender, CMachineStateEventArgs e)
		{	// At the init stage we draw the "header" line on the console
			if (e.MachineState.StepIndex == 0)
			{
				StringBuilder sb = new StringBuilder();
				sb.Append("#STEP");
				sb.Append(" ");
				sb.Append("STIDX");
				sb.Append(" ");
				sb.Append("".PadRight(12, ' '));
				sb.Append("TAPE");
				Console.WriteLine(sb.ToString());
			}
			// ... and we draw the MachineState
			Console.WriteLine(e.MachineState.GetAsText());
		}
	}
}
