﻿using System;
using System.Collections.Generic;

namespace com.scrigner
{
	public static class CContract
	{
		



		// ========================================================================================================
		// Assert non-generic : delega ai metodi generici e throwa CContractException
		// ========================================================================================================
		public static void Assert(bool bResult, string sMessage)
		{
			CContract.Assert<CContractException>(bResult, sMessage);
		}
		public static void Assert(bool bResult, string sConditionText, string sMessage)
		{
			CContract.Assert<CContractException>(bResult, sConditionText, sMessage);
		}
		public static void Assert(bool bResult, string sConditionText, string sMessageTemplate, object[] MessageTemplateArgs)
		{
			CContract.Assert<CContractException>(bResult, sConditionText, sMessageTemplate, MessageTemplateArgs);
		}
		public static void Assert(bool bResult, string sConditionText, string sMessageTemplate, CContractInfo[] infoes)
		{
			CContract.Assert<CContractException>(bResult, sConditionText, sMessageTemplate, infoes);
		}
		// ========================================================================================================
		// Assert con Exception specifica
		// ========================================================================================================
		public static void Assert<EX>(bool bResult, string sMessage)
			where EX : Exception
		{
			CContract.Assert<EX>(bResult, "", sMessage);
		}
		public static void Assert<EX>(bool bResult, string sConditionText, string sMessage)
			where EX : Exception
		{
			CContract.Assert<EX>(bResult, sConditionText, sMessage, new object[]{});
		}
		public static void Assert<EX>(bool bResult, string sConditionText, string sMessageTemplate, object[] MessageTemplateArgs)
			where EX : Exception
		{
			List<CContractInfo> list = new List<CContractInfo>();
			for (int i=0;i<MessageTemplateArgs.Length;i++)
			{
				list.Add(new CContractInfo(i.ToString(), MessageTemplateArgs[i]));
			}
			CContract.Assert<EX>(bResult, sConditionText, sMessageTemplate, list.ToArray());
		}

		// NEW 
		public static void Assert<EX>(bool bResult, string sConditionText, string sMessageTemplate, CContractInfo[] infoes)
			where EX : Exception 
		{
			if (!bResult)
			{	
//				// Costruzione del contract state
//				CContractState cs = new CContractState(infoes);
//				// Composizione del messaggio dell'eccezione
				string sMessage = sMessageTemplate; // cs.ComposeMessage(sConditionText, sMessageTemplate);
				// Se sto throwand la CContractException ....
				if (typeof(EX) == typeof(CContractException))
				{	// ... la throwo ...
					throw new CContractException(sMessage);
				}
				// ... se invece è un'altra eccezione
				else
				{	// .. la genero come InnerExeption ...
					Exception ex = (Exception)Activator.CreateInstance(typeof(EX), new object[]{sMessage});
					// E throwo comunque la ContractException
					throw new CContractException(sMessage, ex);		// ATTENZONE CHE ON MI PIACE : SI THROW SEMPRE UNA CONTRACT EXCEPTION !!!
				}
			}
		}

/*
		public static void ASSERTTEST(Func<bool> test)
		{
			if (!test.Invoke())
			{
				Expression<Func<bool>>.Lambda(test);

				throw new CContractException(new Expression<Func<bool>>(
			}
		}*/

		// ========================================================================================================
		// AssertIsSealed
		// ========================================================================================================
		public static void AssertIsSealed(Type tPivot)
		{
			CContract.AssertIsSealed<CContractException>(tPivot);
		}
		public static void AssertIsSealed<EX>(Type tPivot)
			where EX : Exception 
		{
			CContract.Assert<EX>(tPivot.IsSealed, "tPivot=[" + tPivot.FullName + "].IsSealed()");
		}
		// ========================================================================================================
		// AssertIsConcrete
		// ========================================================================================================
		public static void AssertIsConcrete(Type tPivot)
		{
			CContract.AssertIsConcrete<CContractException>(tPivot);
		}
		public static void AssertIsConcrete<EX>(Type tPivot)
			where EX : Exception 
		{
			CContract.Assert<EX>(tPivot.IsValueType || (tPivot.IsClass || !tPivot.IsAbstract), "tPivot.IsValueType || (tPivot.IsClass || !tPivot.IsAbstract)");
		}

		// ========================================================================================================
		// AssertIsNotEmpty (su stringhe)
		// ========================================================================================================
		public static void AssertIsNotEmpty(string sName, string sValue) 
		{
			CContract.AssertIsNotEmpty<CContractException>(sName, sValue);
		}
		public static void AssertIsNotEmpty<EX>(string sName, string sValue) 
			where EX : Exception 
		{
			CContract.Assert<EX>(("" + sValue).Length > 0, "", "[{1}] is empty.", new CContractInfo[]{new CContractInfo("1", sName)});
		}
		// ========================================================================================================
		// AssertIsNull
		// ========================================================================================================
		public static void AssertIsNull(string sName, object oValue)
		{
			CContract.AssertIsNull<CContractException>(sName, oValue);
		}
		public static void AssertIsNull<EX>(string sName, object oValue) where EX : Exception 
		{
			CContract.Assert<EX>(oValue == null, "", "[{1}] is not null.", new CContractInfo[]{new CContractInfo("1", sName)});
		}

		// ========================================================================================================
		// AssertIsNotNull
		// ========================================================================================================
		public static void AssertIsNotNull(string sName, object oValue)
		{
			CContract.AssertIsNotNull<CContractException>(sName, oValue);
		}
		public static void AssertIsNotNull<EX>(string sName, object oValue) where EX : Exception 
		{
			CContract.Assert<EX>(oValue != null, "", "[{1}] is null.", new CContractInfo[]{new CContractInfo("1", sName)});
		}
		// ========================================================================================================
		// IsTypeOf
		// ========================================================================================================
		public static void AssertIsInstanceOfType<T>(object oInstance)
		{
			CContract.AssertIsInstanceOfType<T, CContractException>(oInstance);
		}
		public static void AssertIsInstanceOfType<T, EX>(object oInstance) where EX : Exception 
		{
			CContract.AssertIsNotNull<EX>("oInstance", oInstance);

			CContract.Assert<EX>(oInstance.GetType() == typeof(T), "oInstance.GetType() == typeof(T) - oInstance.GetType=[" + oInstance.GetType().FullName + "] typeof(T)=[" + typeof(T).FullName + "]");

		}
		// ========================================================================================================
		// AssertIsAssignableFrom
		// ========================================================================================================
		public static void AssertIsAssignableFrom(Type tLeft, Type tRight)
		{
			CContract.AssertIsAssignableFrom<CContractException>(tLeft, tRight);
		}
		public static void AssertIsAssignableFrom<EX>(Type tLeft, Type tRight) where EX : Exception 
		{
			CContract.AssertIsNotNull<EX>("tLeft", tLeft);
			CContract.AssertIsNotNull<EX>("tRight", tRight);

//			CContract.Assert<EX>(tLeft.IsAssignableFrom(tRight), "Type=[" + tLeft.FullName + "] is not assignable from Type=[" + tRight.FullName + "]");
			CContract.Assert<EX>(tLeft.IsAssignableFrom(tRight), "", "Type=[{1}] is not assignable from Type=[{2}]", new CContractInfo[]{new CContractInfo("1", tLeft.FullName), new CContractInfo("2", tRight.FullName)});

//			if (!tLeft.IsAssignableFrom(tRight))
//			{
//				throw new CSimpleFactory<EX>().CreateInstance<EX>(new object[]{"Type=[" + tLeft.FullName + "] is not assignable from Type=[" + tRight.FullName + "]"});
//			}
		}
		// ========================================================================================================
		// AssertIsUTC
		// ========================================================================================================
/*		public static void AssertIsUTCDate(string sName, IMaybe<DateTime> dat)
		{
			CContract.AssertIsUTCDate<CContractException>(sName, dat);
		}
		public static void AssertIsUTCDate<EX>(string sName, IMaybe<DateTime> dat) where EX : Exception 
		{
			if (dat.HasValue())
			{
				CContract.AssertIsUTCDate(sName, dat.GetValue());
			}
		}
	
		public static void AssertIsUTCDate(string sName, DateTime dat)
		{
			CContract.AssertIsUTCDate<CContractException>(sName, dat);
		}
		public static void AssertIsUTCDate<EX>(string sName, DateTime dat) where EX : Exception 
		{
			CContract.Assert<EX>(dat.Kind == DateTimeKind.Utc, "", "Date is Not UTC", new CContractInfo[]{new CContractInfo(sName, dat)});// Date=[" + sName  + "] Value=[" + dat.ToString("yyyy-MM-dd HH:mm:ss") + "] is not UTC. Kind=[" + dat.Kind.ToString() + "]");
		}

		// ========================================================================================================
		// AssertIsLocal
		// ========================================================================================================
		public static void AssertIsLocalDate(string sName, IMaybe<DateTime> dat)
		{
			CContract.AssertIsLocalDate<CContractException>(sName, dat);
		}
		public static void AssertIsLocalDate<EX>(string sName, IMaybe<DateTime> dat) where EX : Exception 
		{
			if (dat.HasValue())
			{
				CContract.AssertIsLocalDate(sName, dat.GetValue());
			}
		}
	
		public static void AssertIsLocalDate(string sName, DateTime dat)
		{
			CContract.AssertIsLocalDate<CContractException>(sName, dat);
		}
		public static void AssertIsLocalDate<EX>(string sName, DateTime dat) where EX : Exception 
		{
			CContract.Assert<EX>(dat.Kind == DateTimeKind.Local, "", "Date is Not Local", new CContractInfo[]{new CContractInfo(sName, dat)});// Date=[" + sName  + "] Value=[" + dat.ToString("yyyy-MM-dd HH:mm:ss") + "] is not UTC. Kind=[" + dat.Kind.ToString() + "]");
		}



		// ========================================================================================================
		// AssertIsUTC
		// ========================================================================================================
//		public static void AssertStringContainsChar(string sText, cha DateTime dat)
//		{
//			CContract.AssertIsUTCDate<CContractException>(sName, dat);
//		}

		// ========================================================================================================
		// AssertIsJust, AssertIsNothing
		// ========================================================================================================
		public static void AssertIsJust(string sName, IMaybe value)
		{
			CContract.AssertIsJust<CContractException>(sName, value);
		}
		public static void AssertIsJust<EX>(string sName, IMaybe value) where EX : Exception 
		{
			if (!(value is IJust))
			{
				CContract.Assert<EX>(!(value is IJust), "", "Value is NOT IJust", new CContractInfo[]{new CContractInfo(sName, value)});
			}
		}

		public static void AssertIsNothing(string sName, IMaybe value)
		{
			CContract.AssertIsNothing<CContractException>(sName, value);
		}
		public static void AssertIsNothing<EX>(string sName, IMaybe value) where EX : Exception 
		{
			if (!(value is INothing))
			{
				CContract.Assert<EX>(!(value is INothing), "", "Value is NOT INothing", new CContractInfo[]{new CContractInfo(sName, value)});
			}
		}
	
	*/

		// ========================================================================================================
		// 20150917 NEW : NON USATO, SOLO UNA PROVA
		// ========================================================================================================
//		public static PreCondition(System.Linq.Expressions.Expression<Func<bool>> expression)
//		{
//			expression.Compile().Invoke(
//		
//		}
//		public static void PreCondition(Func<bool> evaluation)//, string sMessage)
//		{
//			
//			//System.Linq.Expressions.Expression<Func<bool>> exp = evaluation;
//
//			// Calcolo il Message a partire dall'expression
//			System.Linq.Expressions.Expression<Func<int>> exp = 
//				System.Linq.Expressions.Expression.Lambda<Func<int>>(System.Linq.Expressions.Expression.Call(evaluation.Method));
//
//			string sExpressionBody = ((System.Linq.Expressions.LambdaExpression)exp).Body.ToString(); 
//			
//	
//				//expr.Body.ToString()
//			CContract.Assert<CContractException>(bResult, sMessage);
//		}

		/*
		public static TRESULT Return<TRESULT>(bool bCondition, string sConditionText, TRESULT returnValue) // IEnumerable<Func<TRESULT, bool>> PostConditions)
		{
			CContract.AssertIsNotNull("bCondition", bCondition);
			CContract.AssertIsNotNull("sConditionText", sConditionText);
			CContract.AssertIsNotNull("returnValue", returnValue);

			if (bCondition)
			{
				return returnValue;
			}
			else
			{
				throw new CPostConditionException(sConditionText + "returnValue=[" + returnValue.ToString() + "]");
			}
		}
		private static int TestReturnInt()
		{
			int iResult = 25;

			return CContract.Return<int>(iResult > 0 && iResult < 1000, "", iResult);
		}
		// Esattemente come Assert (che si dovrebbe chiamare PreCondition) solo che lancia una InvariantException invece che una PreConditionException ??
		//public static void Invariant(bool bCondition,*/

	}

}

