﻿
namespace com.scrigner.RAMModel
{
	// We should call DataPosition and StatementPo

	public class CStatementCondition : CAbstractStatement 
	{
		// uint vs int, ulong vs long

		private readonly int ziLeftDataPosition;
		private readonly int ziRightDataPosition;
		private readonly int ziNextStatementPositionWhenTrue;		// Index of Next Program Instruction

		// CTOR overload for UNCONDITIONAL branching
		public CStatementCondition(int nextStatementPositionWhenTrue) : this (1, 1, nextStatementPositionWhenTrue) {}
		public CStatementCondition(int leftDataPosition, int rightDataPosition, int nextStatementPositionWhenTrue)
		{
			CContract.Assert(leftDataPosition >= 1, "leftDataPosition >= 1");
			CContract.Assert(rightDataPosition >= 1, "rightDataPosition >= 1");
			CContract.Assert(nextStatementPositionWhenTrue >= 1, "nextStatementPositionWhenTrue >= 1");

			ziLeftDataPosition = leftDataPosition;
			ziRightDataPosition = rightDataPosition;
			ziNextStatementPositionWhenTrue = nextStatementPositionWhenTrue;
		}
		public int LeftDataPosition				{get{return ziLeftDataPosition;}}
		public int RightDataPosition				{get{return ziRightDataPosition;}}
		public int NextStatementPositionWhenTrue	{get{return ziNextStatementPositionWhenTrue;}}

		protected override IStatementResult DoExecute(IMachine machine)
		{	// To avoid two reads from tapes and a comparison, we test if Left==Right
			if (ziLeftDataPosition == ziRightDataPosition)
			{
				return new CStatementResult(ziNextStatementPositionWhenTrue);
				//machine.BranchTo(ziNextStatementPositionWhenTrue);
			}
			else
			{
				long leftValue = machine.Tape.GetValue(ziLeftDataPosition);
				long rightValue = machine.Tape.GetValue(ziRightDataPosition);

				if (leftValue == rightValue)
				{
					//machine.BranchTo(ziNextStatementPositionWhenTrue);
					return new CStatementResult(ziNextStatementPositionWhenTrue);
				}
				else
				{
					return new CStatementResult();
					// the BranchToNext is NOT implemented inside the instruction but it is a constant machine behaviour
				}
			}
		}

		public override string GetAsText()
		{
			return "c(" + ziLeftDataPosition + "," + ziRightDataPosition + "," + ziNextStatementPositionWhenTrue + ")";
		}

	}
}

