﻿
namespace com.scrigner.RAMModel
{
	public class CStatementTransfer : CAbstractStatement 
	{
		private readonly int ziSourcePosition;
		private readonly int ziDestinationPosition;

		public CStatementTransfer(int sourcePosition, int destinationPosition)
		{
			CContract.Assert(sourcePosition >= 1, "sourcePosition >= 1");
			CContract.Assert(destinationPosition >= 1, "destinationPosition >= 1");

			ziSourcePosition = sourcePosition;
			ziDestinationPosition = destinationPosition;
		}
		public int SourcePosition		{get{return ziSourcePosition;}}
		public int DestinationPosition	{get{return ziDestinationPosition;}}

		protected override IStatementResult DoExecute(IMachine machine)
		{	// Here we violate Demeter's Law but we don't care ...
			
			if (ziSourcePosition != ziDestinationPosition)
			{
				long sourceValue = machine.Tape.GetValue(ziSourcePosition);

				machine.Tape.SetValue(sourceValue, ziDestinationPosition);
			}
			return new CStatementResult();
		}
		public override string GetAsText()
		{
			return "t(" + ziSourcePosition + "," + ziDestinationPosition + ")";
		}

	}
}
